
export default function downloadCSV(titleFile, dataCSV, dataType) {

    let newTitleFile, content

    if (!(dataCSV !== null) && !(titleFile !== null)) {
        return false
    }

    newTitleFile = sanitizeCSVName(titleFile)

    switch(dataType){
        case 'Arrays':
            content = convertArraysToString(dataCSV)
            break
        case 'Objects':
            content = convertObjectsToString(dataCSV)
            break
        default:
            content = dataCSV
            break
    }

    let hiddenElement = document.createElement('a')
    hiddenElement.href = 'data:text/csv;charset=UTF-8,' + '\uFEFF' + encodeURI(content)
    hiddenElement.target = '_blank'
    hiddenElement.download = `${newTitleFile}`
    hiddenElement.click()

}

const sanitizeCSVName = (name) => {

    if (/^[A-Za-z0-9]+\.csv$/.test(name)) return name

    if (/^[A-Za-z0-9]+/.test(name)) return name + '.csv'
    else return "file.csv"

}

const convertArraysToString = (data) => {
    let arraysToString = ""
    let header = ""
    let content = ""
    for (let i = 0; i<data.length; i++){
        if(i === 0) {
            for (let a = 0; a<data[i].length; a++){
                if(a===0){
                    header+= data[i][a].charAt(0).toUpperCase() + data[i][a].slice(1)
                } else{
                    header+= ";" + data[i][a].charAt(0).toUpperCase() + data[i][a].slice(1)
                }
            }
            header += "\n"
            arraysToString += header
        } else{
            for (let a = 0; a<data[i].length; a++){
                if(a===0){
                    content+= data[i][a]
                } else{
                    content+= ";" + data[i][a]
                }
            }
            arraysToString += content + "\n"
            content = ""
        }
    }
    return arraysToString
}

const convertObjectsToString = (data) => {
    let header = ""
    let line = ""
    let objectToString = ""
    data.map((obj, index) => {
        if(index === 0){
            header = obtainHeader(obj)
            objectToString += header + "\n"
        }
        line = obtainLine(obj)
        objectToString += line + "\n"
    })
    return objectToString
}

const obtainHeader = (obj) => {
    let count = 0
    let header = ""
    for (let prop in obj) {
        if (count === 0) {
            header += prop.toString().charAt(0).toUpperCase() + prop.toString().slice(1)
        } else {
            header += ";" + prop.toString().charAt(0).toUpperCase() + prop.toString().slice(1)
        }
        count++
    }
    return header
}

const obtainLine = (obj) => {
    let count = 0
    let line = ""
    for (let prop in obj) {
        if (count === 0) {
            line += obj[prop]
        } else {
            line += ";" + obj[prop]
        }
        count++
    }
    return line
}